/*
 * =====================================================================================
 *
 *       Filename:  enc_test.c
 *
 *    Description:
 *
 *        Version:  1.0
 *        Created:  2020年04月23日 15时34分54秒
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  YOUR NAME (),
 *        Company:
 *
 * =====================================================================================
 */

#include <linux/debugfs.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/memblock.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/videodev2.h>
#include <linux/vmalloc.h>
#include <video/sunxi_display2.h>

#include "../../../../video/fbdev/sunxi/disp2/disp/de/disp_manager.h"
#include "../../../../video/fbdev/sunxi/disp2/disp/dev_disp.h"
#include "log.h"
#include "vdecoder.h"

#define CONFIG_ANIMATION_MEM_RESERVE
#define ANIMATION_SIZE 6 * 1024 * 1024
#define ANIMATION_FPS 30
// #define DEBUG_DECODE

static char *animation_file = "animation.fex";
static int pic_width = 0;
static int pic_height = 0;
#ifdef DEBUG_DECODE
static int save_file = 1;
module_param(save_file, int, 0644);
#endif

module_param(animation_file, charp, 0644);
module_param(pic_width, int, 0644);
module_param(pic_height, int, 0644);

struct context_t {
	int frame_size;
	char *ani_buf;
	char *ani_frame_buf;
	struct work_struct work;
	VideoDecoder *vdec;
};
static struct context_t *context;

int preview_animation(unsigned long maddr0, int enabled)
{
	struct disp_manager *mgr;
	struct disp_layer_config my_config;
	struct disp_layer_config *config = &my_config;
	int width = 0;
	int height = 0;

	mgr = disp_get_layer_manager(0);
	if (!mgr || !mgr->force_set_layer_config) {
		loge("preview update error %s: %d", __func__, __LINE__);
		return -1;
	}
	if (!mgr->device || !mgr->device->is_enabled(mgr->device)) {
		loge("[decode] %s: %d", __func__, __LINE__);
		return -1;
	}
	if (mgr && mgr->device && mgr->device->get_resolution)
		mgr->device->get_resolution(mgr->device, &width, &height);
	else {
		loge("preview update error %s: %d", __func__, __LINE__);
		return -1;
	}
	memset(config, 0, sizeof(struct disp_layer_config));
	config->info.fb.addr[0] = maddr0;
	config->info.fb.addr[1] = maddr0 + (pic_width * pic_height);
	config->info.fb.addr[2] = 0;
	config->info.fb.size[0].width = pic_width;
	config->info.fb.size[0].height = pic_height;
	config->info.fb.size[1].width = pic_width / 2;
	config->info.fb.size[1].height = pic_height / 2;
	config->info.fb.size[2].width = 0;
	config->info.fb.size[2].height = 0;
	config->info.fb.align[1] = 0;
	config->info.fb.align[2] = 0;
	config->info.screen_win.width = width;
	config->info.screen_win.height = height;
	config->info.screen_win.x = 0;
	config->info.screen_win.y = 0;
	config->info.fb.crop.x = ((long long)(config->info.screen_win.x) << 32);
	config->info.fb.crop.y = ((long long)(config->info.screen_win.y) << 32);
	config->info.fb.crop.width = ((long long)(pic_width) << 32);
	config->info.fb.crop.height = ((long long)(pic_height) << 32);

	config->channel = 0;
	config->layer_id = 0;
	config->enable = enabled;
	config->info.fb.format = DISP_FORMAT_YUV420_SP_VUVU;
	config->info.mode = LAYER_MODE_BUFFER;
	config->info.zorder = 20;
	config->info.alpha_mode = 1;
	config->info.alpha_value = 0xff;
	mgr->set_layer_config(mgr, config, 1);
	return 0;
}

#ifndef CONFIG_ANIMATION_MEM_RESERVE
static int read_file(const char *path, char *buff, int count, loff_t off)
{
	struct file *fp;
	size_t t = 0;
	mm_segment_t old_fs;

	if ((path == NULL) || (buff == NULL)) {
		pr_err("%s: invalid arguments.\n", __func__);
		return -EFAULT;
	}

	fp = filp_open(path, O_RDONLY, 0);
	if (IS_ERR(fp)) {
		pr_err("open %s# error.\n", path);
		return -EFAULT;
	}

	old_fs = get_fs();
	set_fs(KERNEL_DS);
	t = vfs_read(fp, buff, count, &off);
	vfs_fsync(fp, 0);
	set_fs(old_fs);

	filp_close(fp, NULL);

	return 0;
}

#endif

#ifdef DEBUG_DECODE
static int write_file(const char *path, const char *buff, int count, loff_t off)
{
	struct file *fp;
	size_t t = 0;
	mm_segment_t old_fs;

	if ((path == NULL) || (buff == NULL)) {
		pr_err("%s: invalid arguments.\n", __func__);
		return -EFAULT;
	}

	fp = filp_open(path, O_RDWR | O_CREAT, 0644);
	if (IS_ERR(fp)) {
		pr_err("open or create %s error.\n", path);
		return -EFAULT;
	}

	old_fs = get_fs();
	set_fs(KERNEL_DS);
	t = vfs_write(fp, buff, count, &off);
	vfs_fsync(fp, 0);
	set_fs(old_fs);

	filp_close(fp, NULL);

	return 0;
}
#endif	// DEBUG_DECODE

#ifdef CONFIG_ANIMATION_MEM_RESERVE
static void *map_kernel_cache(unsigned long phys_addr, unsigned long size)
{
	int npages = PAGE_ALIGN(size) / PAGE_SIZE;
	struct page **pages = vmalloc(sizeof(struct page *) * npages);
	struct page **tmp = pages;
	struct page *cur_page = phys_to_page(phys_addr);
	pgprot_t pgprot;
	void *vaddr = NULL;
	int i;

	if (!pages)
		return NULL;

	for (i = 0; i < npages; i++) *(tmp++) = cur_page++;

	pgprot = PAGE_KERNEL;
	vaddr = vmap(pages, npages, VM_MAP, pgprot);

	vfree(pages);
	return vaddr;
}

static u32 animation_reserve_size = 0;
static u32 animation_reserve_base = 0;
static int __init animation_reserve_mem(bool reserve)
{
	if (!animation_reserve_size || !animation_reserve_base)
		return -EINVAL;
	if (reserve)
		memblock_reserve(animation_reserve_base, animation_reserve_size);
	else {
		memblock_free(animation_reserve_base, animation_reserve_size);
		__memblock_free_late(animation_reserve_base, animation_reserve_size);
	}
	return 0;
}

static int __init early_animation_reserve(char *str)
{
	u32 temp[3] = {0};

	if (!str)
		return -EINVAL;
	get_options(str, 3, temp);
	animation_reserve_base = temp[1];
	animation_reserve_size = temp[2];
	if (temp[0] != 2 || animation_reserve_size <= 0)
		return -EINVAL;

	animation_reserve_mem(true);
	pr_info("animation reserve base 0x%x ,size 0x%x\n", animation_reserve_base,
			animation_reserve_size);
	return 0;
}
early_param("animation_reserve", early_animation_reserve);
#endif

static void test_decoder(struct work_struct *work)
{
	VConfig VideoConf;
	VideoStreamInfo VideoInfo;
	VideoStreamDataInfo dataInfo;
	unsigned long addr0;
	int ret;
	int count = 0;
	char *pInput = NULL;
	char *pInput2 = NULL;
	char *pOutput = NULL;
	int pInputLen;
	int pInput2Len;
	VideoPicture *pPicture = NULL;
	int PicSize;
	int data_count = 0;
	int data_size = 0;
	char *p_buffer = context->ani_buf;

	memset(&VideoConf, 0, sizeof(VConfig));
	memset(&VideoInfo, 0, sizeof(VideoStreamInfo));

	context->vdec = CreateVideoDecoder();
	if (context->vdec == NULL) {
		printk(KERN_ERR "nkx_debug:create decoder failed");
		goto __fail;
	}

	VideoInfo.eCodecFormat = VIDEO_CODEC_FORMAT_MJPEG;

	VideoConf.eOutputPixelFormat = PIXEL_FORMAT_NV21;
	logd("nCodecFormat = 0x%x", VideoInfo.eCodecFormat);

	VideoConf.nDeInterlaceHoldingFrameBufferNum = 0;
	VideoConf.nDisplayHoldingFrameBufferNum = 1;
	VideoConf.nRotateHoldingFrameBufferNum = 0;
	VideoConf.nDecodeSmoothFrameBufferNum = 0;
	ret = InitializeVideoDecoder(context->vdec, &VideoInfo, &VideoConf);

	if (ret != 0) {
		loge("decoder demom initialize video decoder fail.");
		goto __fail;
	}

#ifndef CONFIG_ANIMATION_MEM_RESERVE
	ret = read_file(animation_file, p_buffer, ANIMATION_SIZE, 0);
	if (ret < 0) {
		printk(KERN_ERR "nkx_debug:%s:%s:%d\n", __FILE__, __func__, __LINE__);
		goto __fail;
	}
#endif

	data_count = p_buffer[3] << 24 | p_buffer[2] << 16 | p_buffer[1] << 8 |
				 p_buffer[0] << 0;
	p_buffer += 4;
	if (data_count <= 0) {
		printk(KERN_ERR "nkx_debug:%s:%s:%d\n", __FILE__, __func__, __LINE__);
		goto __fail;
	}
	printk(KERN_INFO "nkx_debug:%s:%s:%d picture num:%d\n", __FILE__, __func__,
		   __LINE__, data_count);
	for (count = 0; count < data_count; count++) {
		data_size = p_buffer[3] << 24 | p_buffer[2] << 16 | p_buffer[1] << 8 |
					p_buffer[0] << 0;
		if (data_size <= 0) {
			printk(KERN_ERR "nkx_debug:%s:%s:%d\n", __FILE__, __func__,
				   __LINE__);
			goto __fail;
		}
		context->ani_frame_buf = (char *)&p_buffer[4];
		context->frame_size = data_size;
		p_buffer += 4;
		p_buffer += data_size;

		ret = RequestVideoStreamBuffer(context->vdec, context->frame_size,
									   &pInput, &pInputLen, &pInput2,
									   &pInput2Len, 0);
		if (ret != 0) {
			printk(KERN_ERR
				   "nkx_debug: RequestVideoStreamBuffer fail. request size: %d",
				   pInputLen);
		}
		flush_cache_all();
		memcpy(pInput, context->ani_frame_buf, context->frame_size);

#ifdef DEBUG_DECODE
		if (save_file) {
			msleep(1000);
			msleep(1000);
			msleep(1000);
			msleep(1000);
			msleep(1000);
			msleep(1000);
			printk(KERN_ALERT "nkx_debug:%s:%s:%d\n", __FILE__, __func__,
				   __LINE__);
			write_file("/test.jpg", context->ani_frame_buf,
					   context->frame_size, 0);
		}
#endif	// DEBUG_DECODE
		pInputLen = context->frame_size;
		memset(&dataInfo, 0, sizeof(VideoStreamDataInfo));
		dataInfo.pData = pInput;
		dataInfo.nLength = pInputLen;
		dataInfo.nPts = 0;
		dataInfo.nPcr = 0;
		dataInfo.bIsFirstPart = 1;
		dataInfo.bIsLastPart = 1;
		ret = SubmitVideoStreamData(context->vdec, &dataInfo, 0);
		if (ret < 0) {
			logd("submit error, goto out");
			goto __fail;
		}
		ret = DecodeVideoStream(context->vdec, 0, 0, 0, 0);
		if (ret < 0) {
			loge("encoder error, goto out");
			goto __fail;
		}
		if (ret == VDECODE_RESULT_KEYFRAME_DECODED ||
			ret == VDECODE_RESULT_FRAME_DECODED) {
			//			printk(KERN_ERR "nkx_debug:decoder success");
		} else {
			loge("decoder ret:%d", ret);
			goto __fail;
		}
		pPicture = RequestPicture(context->vdec, 0 /* the major stream*/);
		if (!pPicture) {
			loge("pPicture is null!");
			goto __fail;
		} else {
			addr0 = (unsigned long)pPicture->phyYBufAddr;
			if (pic_width == 0 || pic_height == 0) {
				pic_width = pPicture->nWidth;
				pic_height = pPicture->nHeight;
			}
#ifdef DEBUG_DECODE
			if (save_file) {
				printk(KERN_ALERT "nkx_debug:%s:%s:%d\n", __FILE__, __func__,
					   __LINE__);
				write_file("/test.yuv", (char *)pPicture->pData0,
						   pPicture->nWidth * pPicture->nHeight * 3 / 2, 0);
				save_file = 0;
			}
#endif	// DEBUG_DECODE
			ret = preview_animation(addr0, 1);
			if (ret != 0) {
				goto __fail;
			}
			if (pPicture != NULL) {
				logv("ji****width:%d height:%d p0:%08x p1:%08x addr0:%08x",
					 pPicture->nWidth, pPicture->nHeight, pPicture->pData0,
					 pPicture->pData1, addr0);
				pOutput = pPicture->pData0;
				PicSize = pPicture->nWidth * pPicture->nHeight * 3 / 2;
				ReturnPicture(context->vdec, pPicture);
			}
			msleep(1000 / ANIMATION_FPS);
		}
	}
	printk(KERN_INFO "nkx_debug:%s:%d close animation layer\n", __func__,
		   __LINE__);
__fail:
	preview_animation(addr0, 0);
	if (context->vdec) {
		msleep(1000);
		DestroyVideoDecoder(context->vdec);
	}
	if (context->ani_buf) {
		vfree(context->ani_buf);
	}
	if (context)
		kfree(context);
	context->vdec = NULL;
	context->ani_buf = NULL;
	context = NULL;
}

int dec_debug_register_driver(void)
{
#ifdef CONFIG_ANIMATION_MEM_RESERVE
	char *src_addr = NULL;
	unsigned long map_offset = 0;
#endif

	context = kmalloc(sizeof(struct context_t), GFP_KERNEL);
	if (context == NULL) {
		printk(KERN_ERR "nkx_debug:kmalloc context fail\n");
		return -ENOMEM;
	}
	context->ani_buf = (char *)vmalloc(ANIMATION_SIZE);
	if (context->ani_buf == NULL) {
		printk(KERN_ERR "nkx_debug:kmalloc ani_buf fail\n");
		kfree(context);
		return -ENOMEM;
	}
	context->ani_frame_buf = NULL;
	context->frame_size = 0;

#ifdef CONFIG_ANIMATION_MEM_RESERVE
	if ((animation_reserve_base == 0) || (animation_reserve_size <= 0)) {
		printk(KERN_ERR "nkx_debug:animation_reserve error\n");
		if (context->ani_buf)
			vfree(context->ani_buf);
		if (context)
			kfree(context);
		context->ani_buf = NULL;
		context = NULL;
		return -ENOMEM;
	}
	if (ANIMATION_SIZE < animation_reserve_size) {
		printk(KERN_ERR "nkx_debug:animation_reserve_size too large\n");
		if (context->ani_buf)
			vfree(context->ani_buf);
		if (context)
			kfree(context);
		context->ani_buf = NULL;
		context = NULL;
		return -ENOMEM;
	}

	map_offset = (unsigned long)animation_reserve_base + PAGE_SIZE -
				 PAGE_ALIGN((unsigned long)animation_reserve_base + 1);
	src_addr = (char *)map_kernel_cache(
		(unsigned long)animation_reserve_base - map_offset,
		animation_reserve_size + map_offset);
	if (src_addr == NULL) {
		printk(KERN_ERR "nkx_debug:map_kernel_cache for src_addr failed\n");
		animation_reserve_mem(false);
		if (context->ani_buf)
			vfree(context->ani_buf);
		if (context)
			kfree(context);
		context->ani_buf = NULL;
		context = NULL;
		return -ENOMEM;
	}
	memcpy(context->ani_buf, src_addr + map_offset, animation_reserve_size);
	vunmap(src_addr);
	animation_reserve_mem(false);
#endif

	INIT_WORK(&context->work, test_decoder);
	schedule_work(&context->work);
	return 0;
}

void dec_debug_unregister_driver(void)
{
	kfree(context);
	context = NULL;
}

static int __init dec_test_init(void)
{
	printk(KERN_INFO "animation:%s:%d dec test version 1.0\n", __func__,
		   __LINE__);
	return dec_debug_register_driver();
}

static void __exit dec_test_exit(void) { dec_debug_unregister_driver(); }

fs_initcall(dec_test_init);
module_exit(dec_test_exit);


MODULE_AUTHOR("jilinglin");
MODULE_DESCRIPTION("test case for decoder");
MODULE_LICENSE("GPL");
MODULE_VERSION("1.0alpha");
MODULE_ALIAS("platform:dec_test");