/*
* Copyright (c) 2008-2016 Allwinner Technology Co. Ltd.
* All rights reserved.
*
* File : mjpeg_hal.c
* Description :
* History :
*   Author  : xyliu <xyliu@allwinnertech.com>
*   Date    : 2016/04/13
*   Comment :
*
*
*/
#include <linux/slab.h>
#include "mjpeg_dec.h"
#include "top_reg_def.h"
#include "log.h"

extern volatile u64 uMjpegRegisterBaseAddr;

#define SYS_WriteByte(uAddr, bVal)         {*(volatile u8 *)(uAddr) = (bVal);}
#define SYS_WriteWord(uAddr, wVal)         {*(volatile u16 *)(uAddr) = (wVal);}
#define SYS_WriteDWord(uAddr, dwVal)     {*(volatile u32 *)(uAddr) = (dwVal);}
#define SYS_ReadByte(uAddr)             bVal = (*(volatile u8 *)(uAddr));
#define SYS_ReadWord(uAddr)             wVal = (*(volatile u16 *)(uAddr));
#define SYS_ReadDWord(uAddr)             (*(volatile u32 *)(uAddr));

/* offset */
#define MPEG_REG_o_PIC_HDR          0x00
#define MPEG_REG_o_VOP_HDR          0x04
#define MPEG_REG_o_FRM_SIZE         0x08
#define MPEG_REG_o_PIC_BOUND        0x0C
#define MPEG_REG_o_MB_ADDR          0x10
#define MPEG_REG_o_VE_CTL           0x14
#define MPEG_REG_o_VE_TRIGR         0x18
#define MPEG_REG_o_VE_STAT          0x1C
#define MPEG_REG_o_TRB_TRD          0x20
#define MPEG_REG_o_DIR_MODE         0x24
#define MPEG_REG_o_VLD_START        0x28
#define MPEG_REG_o_VLD_OFFSET       0x2C
#define MPEG_REG_o_VLD_LEN          0x30
#define MPEG_REG_o_VLD_END          0x34
#define MPEG_REG_o_MB_HDR_INF       0x38
#define MPEG_REG_o_FDC_QAC          0x3C
#define MPEG_REG_o_DBLK_ADDR        0x40
#define MPEG_REG_o_NOT_DEC          0x44
#define MPEG_REG_o_RECON_LUMA       0x48
#define MPEG_REG_o_RECON_CHROMA     0x4C
#define MPEG_REG_o_FREF_LUMA        0x50
#define MPEG_REG_o_FREF_CHROMA      0x54
#define MPEG_REG_o_BREF_LUMA        0x58
#define MPEG_REG_o_BREF_CHROMA      0x5C
#define MPEG_REG_o_SOC_X            0x60
#define MPEG_REG_o_SOC_Y            0x64
#define MPEG_REG_o_SOL              0x68
#define MPEG_REG_o_SDLX             0x6C
#define MPEG_REG_o_SDLY             0x70
#define MPEG_REG_o_SSR              0x74
#define MPEG_REG_o_SDCX             0x78
#define MPEG_REG_o_SDCY             0x7C
#define MPEG_REG_o_QMINPUT          0x80
#define MPEG_REG_o_QPINPUT          0x84
#define MPEG_REG_o_RESERVED0        0x88
#define MPEG_REG_o_RESERVED1        0x8C
#define MPEG_REG_o_IQ_IDCT          0x90
#define MPEG_REG_o_MB_HDR           0x94
#define MPEG_REG_o_1ST_MV           0x98
#define MPEG_REG_o_2ND_MV           0x9A
#define MPEG_REG_o_3RD_MV           0xA0
#define MPEG_REG_o_4TH_MV           0xA4
#define MPEG_REG_o_5TH_MV           0xA8
#define MPEG_REG_o_6TH_MV           0xAC
#define MPEG_REG_o_7TH_MV           0xB0
#define MPEG_REG_o_8TH_MV           0xB4
#define MPEG_REG_o_JPG_MCU0         0xB8
#define MPEG_REG_o_JPG_MCU1         0xBC
#define MPEG_REG_o_JPG_RESINT       0xC0
#define MPEG_REG_o_DEC_ERR          0xC4
#define MPEG_REG_o_MB_NUM           0xC8
#define MPEG_REG_o_JPG_YADDR        0xCC
#define MPEG_REG_o_JPG_CADDR        0xD0
#define MPEG_REG_o_ROTAT_CTL        0xD4
#define MPEG_REG_o_JPEG_START_MCU   0xD8
#define MPEG_REG_o_JPEG_END_MCU     0xDC
#define MPEG_REG_o_SRAM_RW_OFFSET   0xE0
#define MPEG_REG_o_SRAM_RW_DATA     0xE4

/* registers */
#define MPEG_REG_PIC_HDR            ( uMjpegRegisterBaseAddr + MPEG_REG_o_PIC_HDR        )
#define MPEG_REG_VOP_HDR            ( uMjpegRegisterBaseAddr + MPEG_REG_o_VOP_HDR        )
#define MPEG_REG_FRM_SIZE           ( uMjpegRegisterBaseAddr + MPEG_REG_o_FRM_SIZE       )
#define MPEG_REG_PIC_BOUND          ( uMjpegRegisterBaseAddr + MPEG_REG_o_PIC_BOUND      )
#define MPEG_REG_MB_ADDR            ( uMjpegRegisterBaseAddr + MPEG_REG_o_MB_ADDR        )
#define MPEG_REG_VE_CTL             ( uMjpegRegisterBaseAddr + MPEG_REG_o_VE_CTL         )
#define MPEG_REG_VE_TRIGR           ( uMjpegRegisterBaseAddr + MPEG_REG_o_VE_TRIGR       )
#define MPEG_REG_VE_STAT            ( uMjpegRegisterBaseAddr + MPEG_REG_o_VE_STAT        )
#define MPEG_REG_TRB_TRD            ( uMjpegRegisterBaseAddr + MPEG_REG_o_TRB_TRD        )
#define MPEG_REG_DIR_MODE           ( uMjpegRegisterBaseAddr + MPEG_REG_o_DIR_MODE       )
#define MPEG_REG_VLD_START          ( uMjpegRegisterBaseAddr + MPEG_REG_o_VLD_START      )
#define MPEG_REG_VLD_OFFSET         ( uMjpegRegisterBaseAddr + MPEG_REG_o_VLD_OFFSET     )
#define MPEG_REG_VLD_LEN            ( uMjpegRegisterBaseAddr + MPEG_REG_o_VLD_LEN        )
#define MPEG_REG_VLD_END            ( uMjpegRegisterBaseAddr + MPEG_REG_o_VLD_END        )
#define MPEG_REG_MB_HDR_INF         ( uMjpegRegisterBaseAddr + MPEG_REG_o_MB_HDR_INF     )
#define MPEG_REG_FDC_QAC            ( uMjpegRegisterBaseAddr + MPEG_REG_o_FDC_QAC        )
#define MPEG_REG_DBLK_ADDR          ( uMjpegRegisterBaseAddr + MPEG_REG_o_DBLK_ADDR      )
#define MPEG_REG_NOT_DEC            ( uMjpegRegisterBaseAddr + MPEG_REG_o_NOT_DEC        )
#define MPEG_REG_RECON_LUMA         ( uMjpegRegisterBaseAddr + MPEG_REG_o_RECON_LUMA     )
#define MPEG_REG_RECON_CHROMA       ( uMjpegRegisterBaseAddr + MPEG_REG_o_RECON_CHROMA   )
#define MPEG_REG_FREF_LUMA          ( uMjpegRegisterBaseAddr + MPEG_REG_o_FREF_LUMA      )
#define MPEG_REG_FREF_CHROMA        ( uMjpegRegisterBaseAddr + MPEG_REG_o_FREF_CHROMA    )
#define MPEG_REG_BREF_LUMA          ( uMjpegRegisterBaseAddr + MPEG_REG_o_BREF_LUMA      )
#define MPEG_REG_BREF_CHROMA        ( uMjpegRegisterBaseAddr + MPEG_REG_o_BREF_CHROMA    )
#define MPEG_REG_SOC_X              ( uMjpegRegisterBaseAddr + MPEG_REG_o_SOC_X          )
#define MPEG_REG_SOC_Y              ( uMjpegRegisterBaseAddr + MPEG_REG_o_SOC_Y          )
#define MPEG_REG_SOL                ( uMjpegRegisterBaseAddr + MPEG_REG_o_SOL            )
#define MPEG_REG_SDLX               ( uMjpegRegisterBaseAddr + MPEG_REG_o_SDLX           )
#define MPEG_REG_SDLY               ( uMjpegRegisterBaseAddr + MPEG_REG_o_SDLY           )
#define MPEG_REG_SSR                ( uMjpegRegisterBaseAddr + MPEG_REG_o_SSR            )
#define MPEG_REG_SDCX               ( uMjpegRegisterBaseAddr + MPEG_REG_o_SDCX           )
#define MPEG_REG_SDCY               ( uMjpegRegisterBaseAddr + MPEG_REG_o_SDCY           )
#define MPEG_REG_QMINPUT            ( uMjpegRegisterBaseAddr + MPEG_REG_o_QMINPUT        )
#define MPEG_REG_QPINPUT            ( uMjpegRegisterBaseAddr + MPEG_REG_o_QPINPUT        )
#define MPEG_REG_RESERVED0          ( uMjpegRegisterBaseAddr + MPEG_REG_o_RESERVED0      )
#define MPEG_REG_RESERVED1          ( uMjpegRegisterBaseAddr + MPEG_REG_o_RESERVED1      )
#define MPEG_REG_IQ_IDCT            ( uMjpegRegisterBaseAddr + MPEG_REG_o_IQ_IDCT        )
#define MPEG_REG_MB_HDR             ( uMjpegRegisterBaseAddr + MPEG_REG_o_MB_HDR         )
#define MPEG_REG_1ST_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_1ST_MV         )
#define MPEG_REG_2ND_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_2ND_MV         )
#define MPEG_REG_3RD_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_3RD_MV         )
#define MPEG_REG_4TH_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_4TH_MV         )
#define MPEG_REG_5TH_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_5TH_MV         )
#define MPEG_REG_6TH_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_6TH_MV         )
#define MPEG_REG_7TH_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_7TH_MV         )
#define MPEG_REG_8TH_MV             ( uMjpegRegisterBaseAddr + MPEG_REG_o_8TH_MV         )
#define MPEG_REG_JPG_MCU0           ( uMjpegRegisterBaseAddr + MPEG_REG_o_JPG_MCU0       )
#define MPEG_REG_JPG_MCU1           ( uMjpegRegisterBaseAddr + MPEG_REG_o_JPG_MCU1       )
#define MPEG_REG_JPG_RESINT         ( uMjpegRegisterBaseAddr + MPEG_REG_o_JPG_RESINT     )
#define MPEG_REG_DEC_ERR            ( uMjpegRegisterBaseAddr + MPEG_REG_o_DEC_ERR        )
#define MPEG_REG_MB_NUM             ( uMjpegRegisterBaseAddr + MPEG_REG_o_MB_NUM         )
#define MPEG_REG_JPG_YADDR          ( uMjpegRegisterBaseAddr + MPEG_REG_o_JPG_YADDR      )
#define MPEG_REG_JPG_CADDR          ( uMjpegRegisterBaseAddr + MPEG_REG_o_JPG_CADDR      )
#define MPEG_REG_ROTAT_CTL          ( uMjpegRegisterBaseAddr + MPEG_REG_o_ROTAT_CTL      )
#define MPEG_REG_JPEGSTART_MCU      ( uMjpegRegisterBaseAddr + MPEG_REG_o_JPEG_START_MCU )
#define MPEG_REG_JPEGSEND_MCU       ( uMjpegRegisterBaseAddr + MPEG_REG_o_JPEG_END_MCU   )
#define MPEG_REG_SRAM_RW_OFFSET     ( uMjpegRegisterBaseAddr + MPEG_REG_o_SRAM_RW_OFFSET )
#define MPEG_REG_SRAM_RW_DATA       ( uMjpegRegisterBaseAddr + MPEG_REG_o_SRAM_RW_DATA   )

static u16 DhtDefault[] = {
    0x0000,0x0000,0x0002,0x000e,0x001e,0x003e,0x007e,0x00fe,
    0x01fe,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,
    0x0000,0x0601,0x0807,0x0a09,0x0c0b,0x0c0c,0x0c0c,0x0c0c,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0004,0x000a,0x001a,0x003a,0x0078,0x00f8,
    0x01f6,0x03f6,0x07f6,0x0ff4,0x1ff0,0x3fe0,0x7fc0,0xff82,
    0x0000,0x0302,0x0906,0x0f0b,0x1712,0x201c,0x2424,0x2524,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0006,0x000e,0x001e,0x003e,0x007e,0x00fe,
    0x01fe,0x03fe,0x07fe,0xffff,0xffff,0xffff,0xffff,0xffff,
    0x0000,0x0403,0x0605,0x0807,0x0a09,0x0c0b,0x0c0c,0x0c0c,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0004,0x000a,0x0018,0x0038,0x0078,0x00f6,
    0x01f4,0x03f6,0x07f6,0x0ff4,0x1ff0,0x3fe0,0x7fc2,0xff88,
    0x0000,0x0302,0x0905,0x100d,0x1b14,0x2420,0x2828,0x2b29,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0100,0x0302,0x0504,0x0706,0x0908,0x0b0a,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0201,0x0003,0x1104,0x1205,0x3121,0x0641,0x5113,0x0761,
    0x7122,0x3214,0x9181,0x08a1,0x4223,0xc1b1,0x5215,0xf0d1,
    0x3324,0x7262,0x0982,0x160a,0x1817,0x1a19,0x2625,0x2827,
    0x2a29,0x3534,0x3736,0x3938,0x433a,0x4544,0x4746,0x4948,
    0x534a,0x5554,0x5756,0x5958,0x635a,0x6564,0x6766,0x6968,
    0x736a,0x7574,0x7776,0x7978,0x837a,0x8584,0x8786,0x8988,
    0x928a,0x9493,0x9695,0x9897,0x9a99,0xa3a2,0xa5a4,0xa7a6,
    0xa9a8,0xb2aa,0xb4b3,0xb6b5,0xb8b7,0xbab9,0xc3c2,0xc5c4,
    0xc7c6,0xc9c8,0xd2ca,0xd4d3,0xd6d5,0xd8d7,0xdad9,0xe2e1,
    0xe4e3,0xe6e5,0xe8e7,0xeae9,0xf2f1,0xf4f3,0xf6f5,0xf8f7,
    0xfaf9,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0100,0x0302,0x0504,0x0706,0x0908,0x0b0a,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0100,0x0302,0x0411,0x2105,0x0631,0x4112,0x0751,0x7161,
    0x2213,0x8132,0x1408,0x9142,0xb1a1,0x09c1,0x3323,0xf052,
    0x6215,0xd172,0x160a,0x3424,0x25e1,0x17f1,0x1918,0x261a,
    0x2827,0x2a29,0x3635,0x3837,0x3a39,0x4443,0x4645,0x4847,
    0x4a49,0x5453,0x5655,0x5857,0x5a59,0x6463,0x6665,0x6867,
    0x6a69,0x7473,0x7675,0x7877,0x7a79,0x8382,0x8584,0x8786,
    0x8988,0x928a,0x9493,0x9695,0x9897,0x9a99,0xa3a2,0xa5a4,
    0xa7a6,0xa9a8,0xb2aa,0xb4b3,0xb6b5,0xb8b7,0xbab9,0xc3c2,
    0xc5c4,0xc7c6,0xc9c8,0xd2ca,0xd4d3,0xd6d5,0xd8d7,0xdad9,
    0xe3e2,0xe5e4,0xe7e6,0xe9e8,0xf2ea,0xf4f3,0xf6f5,0xf8f7,
    0xfaf9,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
};

static void MjpegPrintRegister(MjpegDecodeContext* pMjpegContext)
{

    u32 i = 0;
    u32* reg_base = NULL;

#if 1
    loge("**************top register\n");

    reg_base = (u32*)ve_get_group_reg_addr(pMjpegContext->vconfig.veOpsS,
                                               REG_GROUP_VETOP);

    for(i=0;i<16;i++)
    {
        loge("%02x, 0x%08x 0x%08x 0x%08x 0x%08x \n",
             i, reg_base[4*i],reg_base[4*i+1],reg_base[4*i+2],reg_base[4*i+3]);
    }
    loge("\n");
#endif

    reg_base = (u32*)ve_get_group_reg_addr(pMjpegContext->vconfig.veOpsS,
                                               REG_GROUP_MPEG_DECODER);

    for(i=0;i<16;i++)
    {
        loge("%02x, 0x%08x 0x%08x 0x%08x 0x%08x \n",
                i, reg_base[4*i],reg_base[4*i+1], reg_base[4*i+2],reg_base[4*i+3]);
    }
    logd("  ");
    logd("  ");
    logd("  ");
}

static void set_internal_intr_enable_bits(MjpegDecodeContext* pMjpegContex)
{
    volatile u32* ve_intr_ctrl_reg;
    u64 baseAddr = (u64)ve_get_group_reg_addr(pMjpegContex->vconfig.veOpsS,
                                                REG_GROUP_MPEG_DECODER);
    /* estimate Which video format */
    ve_intr_ctrl_reg = (u32 *)(baseAddr + 0x14);
    *ve_intr_ctrl_reg |= 0x7c;

    return;
}

static void JpegClearStatusReg(MjpegDecodeContext* pMjpegContex)
{
    volatile u32* ve_status_reg;
    u64 baseAddr = (u64)ve_get_group_reg_addr(pMjpegContex->vconfig.veOpsS,
                                                REG_GROUP_MPEG_DECODER);
    ve_status_reg = (u32 *)(baseAddr + 0x1c);
    *ve_status_reg |= 0xf;

    return;
}

s32 SetJpegFormat(JpegDec* jpgctx)
{
    s32 fmt;

    fmt = jpgctx->curCompInfo[0].hSampFactor << 20 |
          jpgctx->curCompInfo[0].vSampFactor << 16 |
          jpgctx->curCompInfo[1].hSampFactor << 12 |
          jpgctx->curCompInfo[1].vSampFactor <<  8 |
          jpgctx->curCompInfo[2].hSampFactor <<  4 |
          jpgctx->curCompInfo[2].vSampFactor;

    jpgctx->maxHSampFactor = MAX(MAX(jpgctx->curCompInfo[0].hSampFactor,
                                        jpgctx->curCompInfo[1].hSampFactor),
                                        jpgctx->curCompInfo[2].hSampFactor);

    jpgctx->maxVSampFactor = MAX(MAX(jpgctx->curCompInfo[0].vSampFactor,
                                        jpgctx->curCompInfo[1].vSampFactor),
                                        jpgctx->curCompInfo[2].vSampFactor);

    if(fmt == 0x211111)
        jpgctx->jpegFormat = JPEG422;
    else if(fmt == 0x221111)
        jpgctx->jpegFormat = JPEG420;
    else if(fmt == 0x121111)
        jpgctx->jpegFormat = JPEG422T;
    else if(fmt == 0x411111)
        jpgctx->jpegFormat = JPEG411;
    else if(fmt == 0x111111)
        jpgctx->jpegFormat = JPEG444;
    else if(fmt == 0x110000)
        jpgctx->jpegFormat = JPEG400;
    else
    {
        jpgctx->jpegFormat = JPEGERR;
        return 0;
    }

    return 1;
}

s32 FillJpegHuffmanTable(JpegDec* jpgctx)
{
    s32                     i;
    JHuffTable*                pJpegHuff;
    JpegComponentInfo*    compptr = 0;
    //u16                     Huffman_Table_Sram[1024];

    if(!jpgctx->hasDht)
    {
        if(!jpgctx->hasConfigDht)
        {    //copy default dht to sramf
            s32* ptr = (s32*)DhtDefault;

            SYS_WriteDWord(MPEG_REG_SRAM_RW_OFFSET, 0);

            for(i=0;i<512;i++)
            {
                SYS_WriteDWord(MPEG_REG_SRAM_RW_DATA, *ptr++);
            }

//            jpgctx->hasConfigDht = 1;
        }

        return 0;//maybe goes here
    }

    for (i = 0; i < jpgctx->nNumComponents; i++)
    {
        compptr = &jpgctx->curCompInfo[i];
        if (compptr->componentId == jpgctx->seqCompId[0])
            break;
    }

    //Y DC
    pJpegHuff = &jpgctx->dcHuffTbl[compptr->dcTblNo];//&mJpegHuffTable[0][dc_index[0]];
    for(i=0;i<16;i++)
        jpgctx->pHuffmanTableSramBuf[i] = pJpegHuff->startCode[i];
    for(i=0;i<16;i+=2)
        jpgctx->pHuffmanTableSramBuf[16+(i>>1)] =
               (pJpegHuff->offset[i+1]<<8) | pJpegHuff->offset[i];
    for(i=0;i<8;i++)
        jpgctx->pHuffmanTableSramBuf[24+i] = 0;
    for(i=0;i<256;i+=2)
        jpgctx->pHuffmanTableSramBuf[512+(i>>1)] =
               (pJpegHuff->symbol[i+1]<<8) | pJpegHuff->symbol[i];

    //Y AC
    pJpegHuff = &jpgctx->acHuffTbl[compptr->acTblNo];
    for(i=0;i<16;i++)
        jpgctx->pHuffmanTableSramBuf[32+i] = pJpegHuff->startCode[i];
    for(i=0;i<16;i+=2)
        jpgctx->pHuffmanTableSramBuf[32+16+(i>>1)] =
               (pJpegHuff->offset[i+1]<<8) | pJpegHuff->offset[i];
    for(i=0;i<8;i++)
        jpgctx->pHuffmanTableSramBuf[32+24+i] = 0;
    for(i=0;i<256;i+=2)
        jpgctx->pHuffmanTableSramBuf[640+(i>>1)] =
                (pJpegHuff->symbol[i+1]<<8) | pJpegHuff->symbol[i];

    for (i = 0; i < jpgctx->nNumComponents; i++)
    {
        compptr = &jpgctx->curCompInfo[i];
        if (compptr->componentId == jpgctx->seqCompId[1])
            break;
    }

    //C DC
    pJpegHuff = &jpgctx->dcHuffTbl[compptr->dcTblNo];
    for(i=0;i<16;i++)
        jpgctx->pHuffmanTableSramBuf[64+i] = pJpegHuff->startCode[i];
    for(i=0;i<16;i+=2)
        jpgctx->pHuffmanTableSramBuf[64+16+(i>>1)] =
                (pJpegHuff->offset[i+1]<<8) | pJpegHuff->offset[i];
    for(i=0;i<8;i++)
        jpgctx->pHuffmanTableSramBuf[64+24+i] = 0;
    for(i=0;i<256;i+=2)
        jpgctx->pHuffmanTableSramBuf[768+(i>>1)] =
               (pJpegHuff->symbol[i+1]<<8) | pJpegHuff->symbol[i];

    //C AC
    pJpegHuff = &jpgctx->acHuffTbl[compptr->acTblNo];
    for(i=0;i<16;i++)
        jpgctx->pHuffmanTableSramBuf[96+i] = pJpegHuff->startCode[i];
    for(i=0;i<16;i+=2)
        jpgctx->pHuffmanTableSramBuf[96+16+(i>>1)] =
                (pJpegHuff->offset[i+1]<<8) | pJpegHuff->offset[i];
    for(i=0;i<8;i++)
        jpgctx->pHuffmanTableSramBuf[96+24+i] = 0;
    for(i=0;i<256;i+=2)
        jpgctx->pHuffmanTableSramBuf[896+(i>>1)] =
                (pJpegHuff->symbol[i+1]<<8) | pJpegHuff->symbol[i];

    for(i=128;i<512;i++)
        jpgctx->pHuffmanTableSramBuf[i] = 0;

    {
        s32* ptr = (s32*)jpgctx->pHuffmanTableSramBuf;

        SYS_WriteDWord(MPEG_REG_SRAM_RW_OFFSET, 0);

        for(i=0;i<512;i++)
        {
            SYS_WriteDWord(MPEG_REG_SRAM_RW_DATA, *ptr++);
        }
    }

    return 0;
}

s32 InitJpegHw(JpegDec* jpgctx, MjpegDecodeContext* pMjpegContext)
{
    s32 i;
    s32 input_data;
    s32 YQuantTbl;
    s32 CQuantTbl;
    u64 tmpaddr = 0;
    struct mem_interface *_memops;

    _memops = pMjpegContext->vconfig.memops;

    SYS_WriteDWord(MPEG_REG_JPG_RESINT, 0 | jpgctx->nRestartInterval)
    //set picture level video engine
    SYS_WriteByte(MPEG_REG_VE_TRIGR+3, (jpgctx->jpegFormat<<3)|3)

    YQuantTbl = jpgctx->curCompInfo[0].quantTblNo;
    CQuantTbl = jpgctx->curCompInfo[1].quantTblNo;

    for(i=0;i<64;i++)
    {
        input_data = (i<<8) | (1<<14);
        input_data |= jpgctx->QTab[YQuantTbl][i];
        SYS_WriteDWord(MPEG_REG_QMINPUT,input_data);
    }

    for(i=0;i<64;i++)
    {
        input_data = (i<<8);
        input_data |= jpgctx->QTab[CQuantTbl][i];
        SYS_WriteDWord(MPEG_REG_QMINPUT,input_data);
    }

    SYS_WriteDWord(MPEG_REG_JPG_YADDR, jpgctx->nRefPicLumaAddr);
    SYS_WriteDWord(MPEG_REG_JPG_CADDR, jpgctx->nRefPicChromaAddr);

    {
        s32 mcuHsize,mcuVsize;

        mcuHsize = jpgctx->maxHSampFactor<<3;
        mcuVsize = jpgctx->maxVSampFactor<<3;
        jpgctx->mcuWidth  = (u16)((jpgctx->nWidth  + mcuHsize - 1) / mcuHsize); //round up
        jpgctx->mcuHeight = (u16)((jpgctx->nHeight + mcuVsize - 1) / mcuVsize); //round up
    }
    input_data = (jpgctx->mcuHeight-1)<<16 | (jpgctx->mcuWidth-1);
    SYS_WriteDWord(MPEG_REG_JPG_MCU0, input_data);
    input_data = (jpgctx->verticalScaleRatio<<2) | (jpgctx->horizontalScaleRatio);
    input_data <<= 8;
    SYS_WriteDWord(MPEG_REG_ROTAT_CTL, input_data);

    FillJpegHuffmanTable(jpgctx);
    tmpaddr = (u64)cdc_mem_get_phy(_memops,
        (void*)(jpgctx->pVbvBase+jpgctx->nVbvSize-1));
    SYS_WriteDWord(MPEG_REG_VLD_END, (u32)(tmpaddr & 0xffffffff));
    return 1;
}



s32 JpegHwDec(JpegDec* jpgctx, MjpegDecodeContext* pMjpegContext)
{
    s32 ret;
    u32 tmp;
    s32 input_data;
    vetop_reglist_t* vetop_reg_list;
    u64 tmpaddr = 0;
    struct mem_interface *_memops;

    _memops = pMjpegContext->vconfig.memops;

    if(jpgctx->bYv12OutFlag==1 || jpgctx->bNv21OutFlag==1)
    {
        SYS_WriteDWord(MPEG_REG_JPG_YADDR, jpgctx->nReconPicLumaAddr);
        SYS_WriteDWord(MPEG_REG_JPG_CADDR, jpgctx->nReconPicChromaAddr);
        if((jpgctx->nVeVersion>= 0x1639) && jpgctx->jpegFormat == JPEG422)
            input_data = 1<<18 | 1<<8 | 8<<4;
        else
            input_data = 1<<8 | 8<<4;
        SYS_WriteDWord(MPEG_REG_VE_CTL,input_data);

        vetop_reg_list = (vetop_reglist_t*)ve_get_group_reg_addr(pMjpegContext->vconfig.veOpsS,
                                                                REG_GROUP_VETOP);
        if(jpgctx->bYv12OutFlag == 1)
        {
            //TODO;
            vetop_reg_list->_e8_chroma_buf_len0.output_data_format = 3;
            //1:32*32 2:YUV 3:YVU Planner Format.
            vetop_reg_list->_e8_chroma_buf_len0.chroma_align_mode = 1;
            //default.0->16 bytes;1->8 bytes.
            vetop_reg_list->_e8_chroma_buf_len0.sdrt_chroma_buf_len =
                    jpgctx->nDispCBufferSize;
        }
        else if(jpgctx->bNv21OutFlag == 1)
        {
            //TODO;
            vetop_reg_list->_e8_chroma_buf_len0.output_data_format = 1;
            //1:32*32 2:YUV 3:YVU Planner Format.
            vetop_reg_list->_ec_pri_output_format.primary_output_format = 0;
            //default 0:32*32 tile
            vetop_reg_list->_ec_pri_output_format.second_special_output_format = 5;
            //0x100:NV12 planner format.0x101:NU12
        }
    }
    else if(jpgctx->enScaleDownOut ==1)
    {
        SYS_WriteDWord(MPEG_REG_JPG_YADDR, jpgctx->nReconPicLumaAddr);
        SYS_WriteDWord(MPEG_REG_JPG_CADDR, jpgctx->nReconPicChromaAddr);

        if((jpgctx->nVeVersion>= 0x1639) && jpgctx->jpegFormat == JPEG422)
            input_data = 1<<18 | 1<<8 | 8<<4;
        else
            input_data = 1<<8 | 8<<4;
        SYS_WriteDWord(MPEG_REG_VE_CTL,input_data);
    }

    if(jpgctx->bRefDispSameFormat==1)
        // add by xyliu for aw1667
    {
        if((jpgctx->nVeVersion>= 0x1639) && ((jpgctx->jpegFormat==JPEG422)
            ||(jpgctx->jpegFormat==JPEG444)))
            input_data = 1<<18 | 1<<8 | 8<<4;
        else
            input_data = 1<<8 | 8<<4;
        SYS_WriteDWord(MPEG_REG_VE_CTL,input_data);

        vetop_reglist_t* vetop_reg_list;
        vetop_reg_list = (vetop_reglist_t*)ve_get_group_reg_addr(pMjpegContext->vconfig.veOpsS,
                                                                REG_GROUP_VETOP);
        vetop_reg_list->_c4_pri_chroma_buf_len.pri_chroma_buf_len = jpgctx->nRefCBufferSize;
        vetop_reg_list->_c8_pri_buf_line_stride.pri_luma_line_stride = jpgctx->nRefYLineStride;
        vetop_reg_list->_c8_pri_buf_line_stride.pri_chroma_line_stride = jpgctx->nRefCLineStride;
        vetop_reg_list->_ec_pri_output_format.primary_output_format =
            jpgctx->eDispFormat;
        vetop_reg_list->_ec_pri_output_format.second_special_output_format =
            jpgctx->eDispFormat;

        vetop_reg_list->_e8_chroma_buf_len0.chroma_align_mode = 1;
        vetop_reg_list->_e8_chroma_buf_len0.luma_align_mode  = 0;
        if((jpgctx->eDispFormat==DISP_FORMAT_YV12) ||
            (jpgctx->eDispFormat==DISP_FORMAT_YU12))
        {
            vetop_reg_list->_e8_chroma_buf_len0.output_data_format = jpgctx->eDispFormat;
        }
        else
        {
            vetop_reg_list->_e8_chroma_buf_len0.output_data_format = 1;
        }

        vetop_reg_list->_e8_chroma_buf_len0.sdrt_chroma_buf_len =
            jpgctx->nDispCBufferSize;

        vetop_reg_list->_cc_sec_buf_line_stride.sec_luma_line_stride = jpgctx->nDispYLineStride;
        vetop_reg_list->_cc_sec_buf_line_stride.sec_chroma_line_stride = jpgctx->nDispCLineStride;
           // jpgctx->nDispCLineStride;
    }

    set_internal_intr_enable_bits(pMjpegContext);

    SYS_WriteDWord(MPEG_REG_VLD_OFFSET, (jpgctx->data - jpgctx->pVbvBase)<<3);
    SYS_WriteDWord(MPEG_REG_VLD_LEN, jpgctx->nFrameSize<<3);
    char* pinput = jpgctx->data;
    tmpaddr = (u64)cdc_mem_get_phy(_memops, (void*)jpgctx->pVbvBase);
    tmp = (u32)(tmpaddr & 0xffffffff);
    tmp = (tmp>>28) | (tmp&0x0ffffff0);
    SYS_WriteDWord(MPEG_REG_VLD_START,  7<<28 | tmp);

    #if 0
    MjpegPrintRegister(pMjpegContext);
    #endif

    SYS_WriteByte(MPEG_REG_VE_TRIGR, 0x0e);

    ret = ve_wait_interrupt(pMjpegContext->vconfig.veOpsS);
	if(ret <= 0)
	{
		loge("ji****wait time out");
	}
    JpegClearStatusReg(pMjpegContext);

    if(ret <= 0)
        return 0;

    return 1;
}

s32 RotatMjpegByHardware(MjpegDecodeContext* pMjpegContex, u32 nInLumaBufAddr,
    u32 nInChromaBufAddr, u32 nOutLumaBufAddr, u32 nOutChromaBufAddr)
{
    JpegDec* jpgctx = (JpegDec*)pMjpegContex->pMjpegDec;
    s32 inputData;
    s32 mcuHsize,mcuVsize;
    u32 mcuWidth = 0;
    u32 mcuHeight = 0;

    SYS_WriteDWord(MPEG_REG_RECON_LUMA, nInLumaBufAddr);
    SYS_WriteDWord(MPEG_REG_RECON_CHROMA,nInChromaBufAddr);

    SYS_WriteDWord(MPEG_REG_JPG_YADDR, nOutLumaBufAddr);
    SYS_WriteDWord(MPEG_REG_JPG_CADDR, nOutChromaBufAddr);

    mcuHsize = jpgctx->maxHSampFactor<<3;
    mcuVsize = jpgctx->maxVSampFactor<<3;

    mcuWidth  =
        (u32)(((jpgctx->nWidth>>jpgctx->horizontalScaleRatio) + mcuHsize - 1) / mcuHsize);
    //round up
    mcuHeight =
        (u32)(((jpgctx->nHeight>>jpgctx->verticalScaleRatio) + mcuVsize - 1) / mcuVsize);
    //round up

    inputData = (mcuHeight-1)<<16 | (mcuWidth-1);
    SYS_WriteDWord(MPEG_REG_JPG_MCU0, inputData);

    inputData = (jpgctx->rotationDegree<=3)? jpgctx->rotationDegree : 0;

    SYS_WriteDWord(MPEG_REG_ROTAT_CTL, inputData);

    inputData = 1<<8 | 5<<4 | 8;
    SYS_WriteDWord(MPEG_REG_VE_CTL,inputData);

    SYS_WriteDWord(MPEG_REG_VE_STAT,0x08);//rotation_finish=1;

    SYS_WriteByte(MPEG_REG_VE_TRIGR+3, (jpgctx->jpegFormat<<3)|5);
    //1633 bug. Mpeg Rotate only VP6 is OK.

    SYS_WriteByte(MPEG_REG_VE_TRIGR, 0x0c);

    ve_wait_interrupt(pMjpegContex->vconfig.veOpsS);

    JpegClearStatusReg(pMjpegContex);
    return 0;
}
