/*
 * =====================================================================================
 *
 *       Filename:  vePolicy.h
 *
 *    Description:  define a array for ve_aw.c, just for ve_aw.c
 *
 *        Version:  1.0
 *        Created:  2020年02月22日 18时15分09秒
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  jilinglin
 *        Company:  allwinnertech
 *
 * =====================================================================================
 */
#ifndef _VE_POLICY_H_
#define _VE_POLICY_H_

typedef enum AwSocName {
    AW_SOC_A10 = 0,
    AW_SOC_A13,
    AW_SOC_A31,
    AW_SOC_A80,
    AW_SOC_A23,
    AW_SOC_C500,
    AW_SOC_A83,
    AW_SOC_H3,
    AW_SOC_H2P,
    AW_SOC_H2,
    AW_SOC_H3S, //10
    AW_SOC_V3,
    AW_SOC_A64,
    AW_SOC_T3,
    AW_SOC_A33,
    AW_SOC_D100,
    AW_SOC_H5,
    AW_SOC_H6,
    AW_SOC_T7,
    AW_SOC_V5,
    AW_SOC_A50, //20
    AW_SOC_V5V200,
    AW_SOC_H3PRO,
    AW_SOC_H616,
    AW_SOC_H313,
    AW_SOC_T503,
    AW_SOC_H513,
    AW_SOC_T507,
    AW_SOC_H517,
    AW_SOC_A100,//29
	AW_SOC_V459,
    AW_SOC_T113I,
} AwSocName;


/* macro for ve version                               *
 * 1855 is ic version, 0x3021000012010 is ve version. *
 * normaly one ve version just for one ic version.    */
#define AW_VE_VERSION_1719_1728 0x3101000012010
#define AW_VE_VERSION_1708      0x3011100020010
#define AW_VE_VERSION_1721      0x3011100021010
#define AW_VE_VERSION_1755      0x3011100012010
#define AW_VE_VERSION_1816      0x3011100021110
#define AW_VE_VERSION_1823      0x3301000012011
#define AW_VE_VERSION_1855      0x3021000012010

struct ve_lock_policy
{
    uint64_t id;
    int32_t sole_venc;
    int32_t sole_jdec;
    //note: the string len maybe long than 20, you should adjust.
    char soc_data[20];
    AwSocName soc_name;
    uint32_t default_freq;
};

static struct ve_lock_policy policy_list[] =
{
    // 1639 enc work alone, so invole nothing;
    // no alone-jpeg-dec, so jpeg-dec
 // {ic-id,       sole_venc, sole_jdec, chip_info, soc_name, default_freq}
    {0x1623,            0,         0,       "00000000",AW_SOC_A10,    300}, //A10
    {0x1625,            0,         0,       "00000000",AW_SOC_A13,    300}, //A13
    {0x1633,            0,         0,       "00000000",AW_SOC_A31,    300}, //A31
    {0x1639,            1,         0,       "00000000",AW_SOC_A80,    300}, //A80
    {0x1650,            0,         0,       "00000000",AW_SOC_A23,    300}, //A23
    {0x1663,            0,         0,       "00000000",AW_SOC_C500,   300}, //C500,F25
    {0x1667,            0,         0,       "00000000",AW_SOC_A33,    300}, //A33,R16
    {0x1673,            0,         0,       "00000000",AW_SOC_A83,    360}, //A83,H8.R58,V66
    {0x1680,            0,         0,       "00000000",AW_SOC_H3,     420}, //H3
    {0x1680,            0,         0,       "00000081",AW_SOC_H3,     420}, //H3
    {0x1680,            0,         0,       "00000042",AW_SOC_H2P,    420}, //H2plus
    {0x1680,            0,         0,       "00000083",AW_SOC_H2P,    420}, //H2plus
    {0x1680,            0,         0,       "H2",      AW_SOC_H2,     420}, //H2
    {0x1680,            0,         0,       "H3s",     AW_SOC_H3S,    420}, //H3s
    {0x1681,            0,         1,       "00000000",AW_SOC_V3,     360}, //V3,V3S,MR100
    {0x1689,            0,         1,       "00000000",AW_SOC_A64,    360}, //A64
    {0x1701,            0,         0,       "00000000",AW_SOC_T3,     360}, //T3,V40,R40
    {0x1707,            0,         0,       "00000000",AW_SOC_D100,   360}, //D100
    {0x1718,            0,         0,       "00000000",AW_SOC_H5,     360}, //H5
//[0:31](enc_ip_version)
//[32:63](dec_ip_version)
    {0x3101000012010,   0,         0,       "00000003",AW_SOC_H6,     552}, //AW1719,AW1728(A63,H6)
    {0x3101000012010,   0,         0,       "00000004",AW_SOC_H6,     552}, //AW1719,AW1728(A63,H6)
    {0x3101000012010,   0,         0,       "00000008",AW_SOC_H6,     552}, //AW1719,AW1728(A63,H6)
    {0x3011100020010,   1,         1,       "00000000",AW_SOC_T7,     300}, //AW1708(T7)
    {0x3011100021010,   1,         1,       "00000000",AW_SOC_V5,     600}, //AW1721(V5)
    {0x3011100012010,   0,         0,       "00000000",AW_SOC_A50,    540}, //AW1755(A50)
    {0x3011100021110,   1,         1,       "00000000",AW_SOC_V5V200, 600}, //AW1816(V5V200)
    {0x3301000012011,   0,         0,       "00000003",AW_SOC_H3PRO,  624}, //H3_pro
    {0x3301000012011,   0,         0,       "00000004",AW_SOC_H3PRO,  624}, //H3_pro
    {0x3301000012011,   0,         0,       "00000008",AW_SOC_H3PRO,  624}, //H3_pro
    {0x3301000012011,   0,         0,       "00005000",AW_SOC_H616,   624}, //H616
    {0x3301000012011,   0,         0,       "00005400",AW_SOC_H616,   624}, //H616
    {0x3301000012011,   0,         0,       "00005c00",AW_SOC_H313,   624}, //H313
    {0x3301000012011,   0,         0,       "00007c00",AW_SOC_T503,   624}, //H503
    {0x3301000012011,   0,         0,       "00002c00",AW_SOC_H513,   624}, //H513
    {0x3301000012011,   0,         0,       "00007400",AW_SOC_T507,   624}, //T507
    {0x3301000012011,   0,         0,       "00002400",AW_SOC_H517,   624}, //H507
    {0x3021000012010,   0,         0,       "00000000",AW_SOC_A100,   600}, //AW1855(A100)
	{0x2111000021210,   1,         1,       "00000000",AW_SOC_V459,   300}, //AW1817(V429)
    {0x1301000010210,   0,         0,       "00003400",AW_SOC_T113I,  576}, //AW1859(F133)
};
#endif
